@@ -11,8 +11,6 @@
 *************************************************************************
 ** Code for testing the client/server version of the SQLite library.
 ** Derived from test4.c.
-**
-** $Id: test7.c,v 1.10 2007/10/05 15:04:13 danielk1977 Exp $
 */
 #include "sqliteInt.h"
 #include "tcl.h"
@@ -22,7 +20,7 @@
 ** the SQLITE_SERVER option.
 */
 #if defined(SQLITE_SERVER) && !defined(SQLITE_OMIT_SHARED_CACHE) && \
-    defined(OS_UNIX) && OS_UNIX && SQLITE_THREADSAFE
+    defined(SQLITE_OS_UNIX) && OS_UNIX && SQLITE_THREADSAFE
 
 #include <stdlib.h>
 #include <string.h>
@@ -118,7 +116,9 @@ static void *client_main(void *pArg){
     p->zErr = 0;
   }
   p->completed++;
+#ifndef SQLITE_OMIT_DEPRECATED
   sqlite3_thread_cleanup();
+#endif
   return 0;
 }
 
@@ -164,7 +164,7 @@ static int tcl_client_create(
   }
   threadset[i].busy = 1;
   sqlite3_free(threadset[i].zFilename);
-  threadset[i].zFilename = sqlite3StrDup(argv[2]);
+  threadset[i].zFilename = sqlite3_mprintf("%s", argv[2]);
   threadset[i].opnum = 1;
   threadset[i].completed = 0;
   rc = pthread_create(&x, 0, client_main, &threadset[i]);
@@ -507,7 +507,7 @@ static int tcl_client_compile(
   client_wait(&threadset[i]);
   threadset[i].xOp = do_compile;
   sqlite3_free(threadset[i].zArg);
-  threadset[i].zArg = sqlite3StrDup(argv[2]);
+  threadset[i].zArg = sqlite3_mprintf("%s", argv[2]);
   threadset[i].opnum++;
   return TCL_OK;
 }
@@ -720,4 +720,4 @@ int Sqlitetest7_Init(Tcl_Interp *interp){
 }
 #else
 int Sqlitetest7_Init(Tcl_Interp *interp){ return TCL_OK; }
-#endif /* OS_UNIX */
+#endif /* SQLITE_OS_UNIX */
